

/*

    Copyright (c) 2017 Motas Electronics Limited (J M HAYES)
    www.motas-synth.uk


    Permission is hereby granted, free of charge, to any person
    obtaining a copy of this software and any associated documentation files
    (the "Software"), to deal in the Software without restriction,
    including without limitation the rights to use, copy, modify, merge,
    publish, distribute, sublicense, and/or sell copies of the Software,
    and to permit persons to whom the Software is furnished to do so,
    subject to the following conditions:

    The above copyright notice and this permission notice shall be
    included in all copies or substantial portions of the Software.

    Any person wishing to distribute modifications to the Software is
    asked to send the modifications to the original developer so that
    they can be incorporated into the canonical version.  This is,
    however, not a binding provision of this license.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
    ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
    CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
    WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *
 * */
#include "readscreen.h"
#include "ui_readscreen.h"
#include <QMessageBox>
#include <QDebug>

#include <QFile>
#include <QFileDialog>

#include <QTimer>
#include "QPainter"
#include <QRect>
#include <QVector>
#include <QProcess>
#include "crc.h"
#include <QThread>

#include"motasfirmware.h"
#include "QMovie"
#include "QGraphicsTextItem"
#include "QGraphicsRectItem"
#include "QPainterPath"
#include "qmath.h"
#define MY_QT_VERSION 5






MotasUtility::MotasUtility(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MotasUtility)
{
    isConnected = false;
     memset(this->oscilloscopeTrace, 2048, sizeof(this->oscilloscopeTrace));
    rawTransferBuffer = NULL;
    reportValues = TRUE;
    this->setWindowIcon(QIcon(":/images/motasUtils.png"));
    ui->setupUi(this);

     ui->pushButtonSend->setIconSize(QSize(32,32));
     ui->pushButtonSendBank->setIconSize(QSize(32,32));
     ui->pushButtonRequestBank->setIconSize(QSize(32,32));
     ui->pushButtonRequest->setIconSize(QSize(32,32));

     ui->pushButtonSendBulk->setIconSize(QSize(32,32));
      ui->pushButton->setIconSize(QSize(32,32));
 ui->pushButtonSendSetup->setIconSize(QSize(32,32));
  ui->pushButtonRequestSetup->setIconSize(QSize(32,32));

    ui->pushButtonSendBank->setStyleSheet("text-align:left;");
    ui->pushButtonRequestBank->setStyleSheet("text-align:left;");
    ui->pushButtonSend->setStyleSheet("text-align:left;");
    ui->pushButtonRequest->setStyleSheet("text-align:left;");
    ui->pushButton->setStyleSheet("text-align:left;");
    ui->pushButtonSendBulk->setStyleSheet("text-align:left;");
    ui->pushButtonSendSetup->setStyleSheet("text-align:left;");
    ui->pushButtonRequestSetup->setStyleSheet("text-align:left;");

    ui->labelIsCorrupted->setVisible(!true);

    ui->labelVersion->setText(VERSION);

    QPixmap logo = QPixmap(":/images/MotasElectronicsLogoMed.png");
    ui->labelLogo->setPixmap(logo.scaledToWidth(120,Qt::SmoothTransformation));
    //midiStreamScreenGrab = new QVector<quint8>();
    midiStreamData1 = new QVector<quint8>();
    this->writeMidiStreamData = this->midiStreamData1;
    this->readMidiStreamData = this->midiStreamData1;
    midiStreamData2 = new QVector<quint8>();
   // this->MIDIbyteCounter = 0;
    this->rescaleFactor = 1;
    this->smallImage = new QImage(128, 64,QImage::Format_RGB32 );

    this->image1 = QImage(rescaleFactor*2, rescaleFactor,QImage::Format_RGB32 );
    this->image2 = QImage(rescaleFactor*2, rescaleFactor,QImage::Format_RGB32 );

    timer = NULL;
    displayedOscilloscopeTrace = FALSE;
    this->oscilloscopeLine = NULL;
    midiIn = NULL;
    midiOut = NULL;
    updateMIDI(true);


    utilities = new Utilities();

    motasFirmware = new MotasFirmware(this);


    fileNameLocationDefault = QString("/");

    ui->pushButtonSave->setEnabled(false);
    ui->pushButtonRequest->setEnabled(false);
    ui->pushButtonSend->setEnabled(false);

    displayedImage = TRUE;

    numBytesReceived = 0;


    rawTransferBuffer = new quint8[LARGEST_BUFFER_LENGTH];

    for (int i = 0; i < BANK_OF_PATCHES_LENGTH; i++)
    {
        QListWidgetItem* item = new QListWidgetItem("1");
        ui->listWidgetPatches->addItem(item);
        bankOfPatches.append(new quint8[PATCH_LENGTH_BYTES_UNPACKED]);
    }

    for (int i = 0; i < BANK_OF_PATTERNS_LENGTH; i++)
    {
        QListWidgetItem* item = new QListWidgetItem("1");
        ui->listWidgetPatterns->addItem(item);
        bankOfPatterns.append(new quint8[PATTERN_LENGTH_BYTES_UNPACKED]);
    }


    for (int i = 0; i < BANK_OF_SEQUENCES_LENGTH; i++)
    {
        QListWidgetItem* item = new QListWidgetItem("1");
        ui->listWidgetSequences->addItem(item);
        bankOfSequences.append(new quint8[SEQUENCE_LENGTH_BYTES_UNPACKED]);
    }




    ui->listWidgetPatches->setCurrentRow(0);
    ui->listWidgetPatterns->setCurrentRow(0);
    ui->listWidgetSequences->setCurrentRow(0);


    clearAll(0);
    clearAll(1);
    clearAll(2);




   // progressMovie = new QMovie(":/images/loadingcirclests16.gif");
     progressMovie = new QMovie(":/images/loadingRed2.gif");
     progressMovieMIDIOut = new QMovie(":/images/loadingRed2.gif");
    ui->labelProgress->setMovie(progressMovie);
    ui->labelProgress->show();
    progressMovie->start();
    progressMovie->setPaused(true);

    ui->labelProgressMIDIOut->setMovie(progressMovieMIDIOut);
    ui->labelProgressMIDIOut->show();
    progressMovieMIDIOut->start();
    progressMovieMIDIOut->setPaused(true);

    MIDItransferState = 0;



    oscilloscopeScene =  new QGraphicsScene(this);;

    QRectF r(0, -2048/OSCILLOSCOPE_TRACE_Y_SCALING, OSCILLOSCOPE_TRACE_NUM_POINTS * OSCILLOSCOPE_TRACE_X_SCALING, 4096/OSCILLOSCOPE_TRACE_Y_SCALING);
    QBrush b(QColor(0,0,0,255));
    QPen p;
    oscilloscopeScene->addRect(r, p, b);


    QPen pen(QColor(100,100,0,255), 1, Qt::DotLine, Qt::RoundCap, Qt::RoundJoin);

    QVector<qreal> dashes;
    qreal space = 8;
    dashes << 2 << space;
    pen.setDashPattern(dashes);
    QLine l(0,0, OSCILLOSCOPE_TRACE_NUM_POINTS * OSCILLOSCOPE_TRACE_X_SCALING,0 );
    QLine ltop(0,1024/OSCILLOSCOPE_TRACE_Y_SCALING, OSCILLOSCOPE_TRACE_NUM_POINTS * OSCILLOSCOPE_TRACE_X_SCALING,1024/OSCILLOSCOPE_TRACE_Y_SCALING );
    QLine lbottom(0,-1024/OSCILLOSCOPE_TRACE_Y_SCALING, OSCILLOSCOPE_TRACE_NUM_POINTS * OSCILLOSCOPE_TRACE_X_SCALING,-1024/OSCILLOSCOPE_TRACE_Y_SCALING);


    this->oscilloscopeScene->addLine(ltop, pen);
    this->oscilloscopeScene->addLine(l, pen);
    this->oscilloscopeScene->addLine(lbottom, pen);


    int width = OSCILLOSCOPE_TRACE_NUM_POINTS * OSCILLOSCOPE_TRACE_X_SCALING;
    QLine lVertical(width / 6,-2048/OSCILLOSCOPE_TRACE_Y_SCALING, width / 6,2048/OSCILLOSCOPE_TRACE_Y_SCALING );
    this->oscilloscopeScene->addLine(lVertical, pen);
    QLine lVertical2(2*width / 6,-2048/OSCILLOSCOPE_TRACE_Y_SCALING, 2* width/ 6,2048/OSCILLOSCOPE_TRACE_Y_SCALING );
    this->oscilloscopeScene->addLine(lVertical2, pen);
    QLine lVertical3(3*width / 6,-2048/OSCILLOSCOPE_TRACE_Y_SCALING, 3* width/ 6,2048/OSCILLOSCOPE_TRACE_Y_SCALING );
    this->oscilloscopeScene->addLine(lVertical3, pen);
    QLine lVertical4(4*width / 6,-2048/OSCILLOSCOPE_TRACE_Y_SCALING, 4* width/ 6,2048/OSCILLOSCOPE_TRACE_Y_SCALING );
    this->oscilloscopeScene->addLine(lVertical4, pen);
    QLine lVertical5(5*width / 6,-2048/OSCILLOSCOPE_TRACE_Y_SCALING, 5* width/ 6,2048/OSCILLOSCOPE_TRACE_Y_SCALING );
    this->oscilloscopeScene->addLine(lVertical5, pen);





    if (timer == NULL)
    {
        timer = new QTimer(this);
        connect(timer, SIGNAL(timeout()), this, SLOT(update()));
        countDownTimerMIDIIn = 0;
        countDownTimerMIDIOut = 0;
        timer->start(80); // set to 12.5 FPS
    }
}

MotasUtility::~MotasUtility()
{


    if (midiOut->isPortOpen())
    {
        midiOut->closePort();
        int ms = 100;
        // wait until MIDI in stops after sending 'stop ' command
        #ifdef Q_OS_WIN
                Sleep(uint(ms));
         #else
                struct timespec ts = { ms / 1000, (ms % 1000) * 1000 * 1000 };
                nanosleep(&ts, NULL);
         #endif
    }

    if (midiIn->isPortOpen())
        midiIn->closePort();

    delete midiIn;

    delete midiOut;
    delete ui;


}

void MotasUtility::setupScreenShotView(ScreenCapture *c)
{
    this->screenCapture = c;
}

void MotasUtility::setUpScopeView(OscilloscopeScreen *o)
{
    this->oscilloscopeCapture = o;
     this->oscilloscopeCapture->setGraphics(oscilloscopeScene);
}



void MotasUtility::updateMIDI(bool restart)
{
    if (restart)
    {
        closePorts();
    }

    unsigned int numPorts = midiIn->getPortCount();
    unsigned int numPortsOut = midiOut->getPortCount();

    ui->MIDIin_comboBox->clear();
    ui->comboBoxOutMIDI->clear();
    if ( numPorts <= 0 )
    {
        QMessageBox::warning(this, "Error", "No MIDI input ports available!");

    } else
    {
        if ( numPortsOut <= 0 )
        {
            QMessageBox::warning(this, "Error", "No MIDI output ports available!");

        }
        else
        {
            for (quint32 i = 0; i < numPorts; i++)
            {
                QString name = QString::fromStdString(midiIn->getPortName(i)).simplified();

                ui->MIDIin_comboBox->addItem(name);

                if (name.contains("Motas"))
                    ui->MIDIin_comboBox->setCurrentIndex(i);
            }

            for (quint32 i = 0; i < numPortsOut; i++)
            {
                QString name = QString::fromStdString(midiOut->getPortName(i)).simplified();

                //ui->comboBoxOutMIDI->addItem(QString::fromStdString(midiOut->getPortName(i)).simplified());

                ui->comboBoxOutMIDI->addItem(name);
                if (name.contains("Motas"))
                    ui->comboBoxOutMIDI->setCurrentIndex(i);
            }
        }
    }
}

static bool notSaving = true;

void  MotasUtility::midiInErrorCallback( RtMidiError::Type type, const std::string &errorText, void* userData)
{
    Q_UNUSED(type);
    Q_UNUSED(userData);
    //QString msg(er)
    QString msg = QString::fromStdString(errorText);
    qDebug() << "MIDI in error: " << msg;
}
void  MotasUtility::midiOutErrorCallback( RtMidiError::Type type, const std::string &errorText, void* userData)
{
    Q_UNUSED(type);
    Q_UNUSED(userData);
    QString msg = QString::fromStdString(errorText);
    qDebug() << "MIDI out error: " << msg;
}
void MotasUtility::mycallback(double deltatime, std::vector< unsigned char > *message, void *userData )
{
     Q_UNUSED(deltatime);
     // cast the void pointer back to the class poiner
     MotasUtility* self = static_cast<MotasUtility*>(userData);
     unsigned int nBytes = message->size();

    for (quint32 i = 0; i < nBytes; i++)
    {
        self->updateMIDIReceiveByte(message->at(i));
    }

     self->countDownTimerMIDIIn = 3;
}


void MotasUtility::updateMIDIreceivedScreenGrab(quint8* buffer)
{
   //qDebug() << " new screen data";
   if (displayedImage) // don't bother to process new image if the previous one hasn't been shown yet
   {
       QImage* i = imageIt(buffer);
       if (i != NULL)
       {
           if (toggleLockImage)
               this->image1 = i->scaledToWidth(this->rescaleFactor * 256,Qt::FastTransformation);
           else
               this->image2 = i->scaledToWidth(this->rescaleFactor * 256,Qt::FastTransformation);
           displayedImage = false;
           toggleLockImage = !toggleLockImage;
       }
   }

}

void MotasUtility::updateOscilloscope(quint8 *buffer)
{

   // qDebug() << " new scope data";
    if (displayedOscilloscopeTrace) // don't bother to process new image if the previous one hasn't been shown yet
    {

        memcpy((void*) this->oscilloscopeTrace, (void*) buffer, OSCILLOSCOPE_TRACE_NUM_POINTS*2);
        displayedOscilloscopeTrace = false;

    }
}


void MotasUtility::on_MIDIopen_pushButton_clicked()
{
    if (isConnected)
    {
        closePorts();
        return;
    }
    ui->pushButtonRequest->setEnabled(false);
    ui->pushButtonSend->setEnabled(false);
    // Call function to select and open port.
    try
    {
        //updateMIDI(true);
        if (this->midiIn->isPortOpen())
        {
            QMessageBox::warning(this, "Error", "Port open!");
            return;
        }
        this->midiIn->setErrorCallback(&this->midiInErrorCallback, NULL);


        midiIn->setCallback(&this->mycallback, this);


        this->midiIn->openPort(ui->MIDIin_comboBox->currentIndex());

        if (this->midiIn->isPortOpen())
        {
          //  QMessageBox::information(this, "Ok", "Port opened");
            qDebug() << "port opened";
        }

        this->midiOut->setErrorCallback(&this->midiOutErrorCallback);
         this->midiOut->openPort(ui->comboBoxOutMIDI->currentIndex());
         this->MIDIReady = true;


        midiIn->setErrorCallback();
        // Don't ignore sysex, do ignore timing, or active sensing messages.
        midiIn->ignoreTypes( false, true, !true );


        //    ui->MIDIopen_pushButton->setEnabled(false);
        // send SYSEX command to start the Motas sending Bitmap data;

        //ui->checkBoxMidiConnected->setChecked(true);
        isConnected = true;

        ui->pushButtonRequest->setEnabled(!false);
        ui->pushButtonSend->setEnabled(false);
        ui->pushButtonRequestBank->setEnabled(true);
        ui->pushButtonSendBank->setEnabled(true);

        //ui->MIDIopen_pushButton->setEnabled(false);

        ui->comboBoxOutMIDI->setEnabled(false);
        ui->MIDIin_comboBox->setEnabled(false);
        ui->MIDIopen_pushButton->setText("Disconnect");
        ui->pushButtonAbort->setEnabled(true);
    }
    catch ( RtMidiError &error )
    {
        QMessageBox::warning(this, "Error", "A problem has occurred attempting to connect to the MIDI port.");
        error.printMessage();
        //ui->checkBoxMidiConnected->setChecked(false);
    }
}

void MotasUtility::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

QImage* MotasUtility::imageIt(quint8* buffer)
{
    if (notSaving)
    {
        int x = 0;
        int y = 0;
        int i = 0;
        for (i = 0; i < 4096; i++)
        {
            for (int n = 0; n < 2; n++)
            {
                quint8 val  = buffer[i];
                quint32 data ;
                if (n == 0)
                {
                    data = (val & 0x0F) ;
                }
                else
                {
                    data = ((val >> 4) & 0x0F) ;
                }
                data <<= 4;

              //  data <<= 4; // since OLED data has only 3bit intensity then can rescale to make brighter
 //qDebug()<< "midi:" << this->midiStream->at(i) << " data: " << data << "n " << n;



                //if (this->ui->compressIntensity_checkBox->isChecked())
                {
                    double intData = data;
                    // compress so dark stuff is not too dark
                    intData *= 1.1 + 1;
                    intData = (int) intData;
                    if (intData > 0xFF)
                        intData = 0xFF;
                    if (intData < 0)
                        intData = 0;
                    data = intData;
                }



                if (data > 0xFF)
                    data = 0xFF;
                QRgb* line = (QRgb*) this->smallImage->scanLine(y);


                {
                    line[x  % 128] = (data << 8) | ((quint32) data << 16);
                }
               // line[x  % 128] = (data) | (data << 8) | ((quint32) data << 16); // white

                x++;
                if ((x % 128) == 0)
                {
                    y++;

                  //  qDebug() << "y: " << y;
                }
            }
        }
    }
    return this->smallImage;
}

void MotasUtility::update()
{

    bool saveImage = ui->checkBoxSaveImages->isChecked();
    QString loc = ui->lineEditSaveImagesLoc->text().trimmed();


    if (saveImage && loc.isEmpty())
    {
        loc = QFileDialog::getSaveFileName(this,"save images");
        ui->lineEditSaveImagesLoc->setText(loc);
    }

    //if (!displayedImage) // always show and save the image
    if (ui->checkBoxSendBitmap->isChecked())
    {
        if (!toggleLockImage)
        {
           //ui->imageLabel->setPixmap(QPixmap::fromImage(this->image1, 0));
            //screenShotLabel->setPixmap(QPixmap::fromImage(this->image1, 0));
            screenCapture->setImage(QPixmap::fromImage(this->image1, 0), saveImage, loc);

        }
        else
        {
          // ui->imageLabel->setPixmap(QPixmap::fromImage(this->image2, 0));
            // screenShotLabel->setPixmap(QPixmap::fromImage(this->image2, 0));
            screenCapture->setImage(QPixmap::fromImage(this->image2, 0), saveImage,loc);
        }
        displayedImage = true;
    }

    if (!displayedOscilloscopeTrace)
    {

        QPolygonF p;



        qint32 max = -2048;
        qint32 min = 2048;

        if (ui->checkBoxOscilloscopeAutoScale->isChecked())
        {
            for (int i = 0; i < OSCILLOSCOPE_TRACE_NUM_POINTS; i++)
            {
                qint16 val = (qint16) this->oscilloscopeTrace[i] - 2048;
                if (max < val)
                    max = val;
                if (min > val)
                    min = val;
            }

            if (max < -min)
                max = -min;

            max = (max * 120) / 100;
            if (max < 64)
                max = 64;

        }

        qint32 smoothedVals[OSCILLOSCOPE_TRACE_NUM_POINTS];
        if (ui->checkBoxOscilloscopeFilterDisplay->isChecked())
        {


            for (int i = 0; i < OSCILLOSCOPE_TRACE_NUM_POINTS - 1; i++)
            {
                qint16 val = (qint16) this->oscilloscopeTrace[i] - 2048;
                qint16 val2 = (qint16) this->oscilloscopeTrace[i + 1] - 2048;

                smoothedVals[i] = (val + val2)/ 2 ;

            }
            smoothedVals[OSCILLOSCOPE_TRACE_NUM_POINTS - 1] = (qint16) this->oscilloscopeTrace[OSCILLOSCOPE_TRACE_NUM_POINTS - 1] - 2048;
        }

        for (int i = 0; i < OSCILLOSCOPE_TRACE_NUM_POINTS; i++)
        {
            qint32 val;
            if (!ui->checkBoxOscilloscopeFilterDisplay->isChecked())
                val = (qint16) this->oscilloscopeTrace[i] - 2048;
            else
                val = smoothedVals[i];


            if (ui->checkBoxOscilloscopeAutoScale->isChecked())
            {
                val = (2048 * val ) / max;
                ui->dialOscilloscopeGain->setEnabled(false);
            }
            else
            {
                ui->dialOscilloscopeGain->setEnabled(true);
                val = val * ui->dialOscilloscopeGain->value();
            }
           p << QPointF(i*OSCILLOSCOPE_TRACE_X_SCALING, val / OSCILLOSCOPE_TRACE_Y_SCALING);
           // p << QPointF(i, 0);


          // qDebug() << this->oscilloscopeTrace[i];
        }


        QPainterPath path;
        path.addPolygon(p);
      //  QGraphicsPathItem contour;
       // contour.setPen(new QPen(QColor.black));



        //QBrush b(QColor(200,255,0,255));
        QBrush b(QBrush(Qt::transparent));

        int brightness = ui->dialOscilloscopeBrightness->value();
        QPen pen(QColor(brightness, brightness,0,255));
        pen.setWidth(2);

        if (this->oscilloscopeLine == NULL)
            this->oscilloscopeLine = oscilloscopeScene->addPath(path, pen, b);
        else
        {
            this->oscilloscopeLine->setPath(path);
            this->oscilloscopeLine->setPen(pen);
        }

        displayedOscilloscopeTrace = true;
    }

    //if (ui->checkBoxShowMIDIactivity->isChecked())
    {
        if (countDownTimerMIDIIn > 0)
        {
             countDownTimerMIDIIn--;
        }
        if (countDownTimerMIDIIn == 0)
            progressMovie->setPaused(true);
        else if (progressMovie->state() == QMovie::Paused)
            progressMovie->start();

        if (countDownTimerMIDIOut > 0)
        {
             countDownTimerMIDIOut--;
        }
        if (countDownTimerMIDIOut == 0)
            progressMovieMIDIOut->setPaused(true);
        else if ( progressMovieMIDIOut->state() == QMovie::Paused)
            progressMovieMIDIOut->start();
    }/* else if (countDownTimerMIDIOut > 0 || countDownTimerMIDIIn > 0)
    {
       countDownTimerMIDIOut = 0;
        countDownTimerMIDIOut = 0;
         progressMovie->setPaused(true);
         progressMovieMIDIOut->setPaused(true);
    }*/

    if (MIDItransferState == 1)
    {
        processMIDIdata(MIDItransferType, readMidiStreamData);
        MIDItransferState = 0;
    }

    resendMIDIcommands();
}


void MotasUtility::memcpyReverse32(char* out, char* in)
{
    for (int i = 0; i < 4; i++)
    {
        out[i] = in[i];
    }
}

void MotasUtility::memcpyForward32(char* out, char* in)
{
    for (int i = 0; i < 4; i++)
    {
        out[i] = in[3 - i];
    }
}


int MotasUtility::sendDataOverMIDIDirect(unsigned char* buffer, quint32 fileLength)
{
    if (buffer == NULL)
        return 0;
    if (this->midiOut->isPortOpen())
    {
        ui->progressBarMidi->setMinimum(0);
        ui->progressBarMidi->setMaximum(fileLength);

        QThread* th = new QThread;

        for (quint32 j = 0; j < fileLength; )
        {
            QVector<quint8> message;
            quint32 i;
            int packetSize = 0;
            for (i = 0; i < fileLength; i++)
            {
                if (i + j >= fileLength)
                    break;
                unsigned char c = buffer[i + j];

                message.append(c);
               // qDebug() << "byte: " << (quint8) c << " i + j: " << i + j;
                if (c == 0xF7)
                {
               //     qDebug() << i + j << "END packet";
                    break;
                }
                packetSize++;
            }
            j += i + 1;
            std::vector<unsigned char> m = message.toStdVector();
            countDownTimerMIDIOut = 6;
            this->midiOut->sendMessage( &m );
            ui->progressBarMidi->setValue(j+1);
            QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
            th->usleep((ui->spinBoxMIDIdelay->value() * packetSize) / 32);
        }
        delete th;
        return 1;
    } else
    {
        QMessageBox::warning(this, "Error", "No MIDI out port!.");
        return 0;

    }
}


void MotasUtility::sendDataOverMIDI(char* buffer, quint32 fileLength, int type, quint8 msg0, quint8 msg1, quint8 msg2)
{
    if (this->midiOut->isPortOpen())
    {
        for (quint32 i = 0; i < fileLength; i++)
        {
            if ((quint8) buffer[i] > 0x7F)
            {
                qDebug() << "ERROR IN FILE  - value greater than 0x7F!";
                QMessageBox::warning(this, "Error", "ERROR IN FILE  - value greater than 0x7F!");
                return;
            }
        }
        //qDebug() << "MidiSettingsDialog: sending firmware";
        quint32 packetSize = 64;
        quint32 lastPacket = 1 + (fileLength / packetSize);
        ui->progressBarMidi->setMinimum(0);
        ui->progressBarMidi->setMaximum(lastPacket);
        //quint32 firmwareBytesSent = 0;
        QThread* th = new QThread;
        for (quint32 j = 0; j < lastPacket; j++)
        {
            QVector<quint8> message = utilities->prepareSysExPacket(j, buffer, packetSize, fileLength, lastPacket, type,
                                                                    msg0, msg1, msg2, ui->spinBoxMotasModel->value(), ui->spinBoxChannel->value() - 1);

            std::vector<unsigned char> m = message.toStdVector();
            countDownTimerMIDIOut = 6;
            this->midiOut->sendMessage( &m );


            ui->progressBarMidi->setValue(j+1);


            // process the event queue - except user input
            QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
            th->usleep((ui->spinBoxMIDIdelay->value() * packetSize) / 64);

        }
        delete th;

        closePorts();

    } else
    {
        QMessageBox::warning(this, "Error", "No MIDI out port!.");
    }
}

void MotasUtility::on_pushButtonSendFirmware_clicked()
{


    //if (!ui->checkBoxMidiConnected->isChecked())
    if (isConnected)
    {
        on_pushButtonFindDevices_clicked();
        on_MIDIopen_pushButton_clicked();
    }

    QString fileName =  QFileDialog::getOpenFileName(this,"select firmware file",  fileNameLocationDefault + "/*." + MOTAS_FIRMARE_FILE_EXTENSION, "*." + MOTAS_FIRMARE_FILE_EXTENSION);

    if (fileName.isEmpty())
        return;


     qDebug() << fileName;

    QFileInfo fileInfo(fileName);
    fileNameLocationDefault = fileInfo.absolutePath();
    qDebug() << fileNameLocationDefault;

     QFile f(fileName);
     f.open(QIODevice::ReadOnly);
     quint32 fileLength = f.size();

     qDebug() << "File length (bytes): " << fileLength;

     if (fileLength > (1024*1024) || (fileLength < (200)))
     {

         qDebug() << "ERROR IN FILE SIZE";
         QMessageBox::warning(this, "Error", "ERROR IN FILE SIZE");
         f.close();
         return;
     }

     char* buffer = new char[fileLength];

     f.read(buffer, fileLength); // read in from the file into the new buffer
     f.close();

     sendDataOverMIDI(buffer, fileLength, MIDI_SYSEX_FIRMWARE, 0, 0, 0);

}


void MotasUtility::closePorts()
{
    try
    {
        if (midiIn != NULL && midiIn->isPortOpen())
        {
            this->midiIn->cancelCallback();
            this->midiIn->closePort();
        }
        if (midiOut != NULL && this->midiOut->isPortOpen())
        {
            this->midiOut->closePort();
            this->MIDIReady = !true;
        }


        //ui->MIDIopen_pushButton->setEnabled(false);
       //ui->checkBoxMidiConnected->setChecked(false);

        isConnected = false;

        if (midiIn != NULL)
        {
            delete midiIn;
            midiIn = NULL;
        }
        if (midiOut != NULL)
        {
            delete midiOut;
            midiOut = NULL;
        }
        try
        {
            midiIn = new RtMidiIn();

            midiOut = new RtMidiOut();
        }

        catch ( RtMidiError &error )
        {
            error.printMessage();
            exit( EXIT_FAILURE );
        }


        ui->MIDIopen_pushButton->setText("Connect");
        ui->pushButtonAbort->setEnabled(!true);
        ui->comboBoxOutMIDI->setEnabled(true);
        ui->MIDIin_comboBox->setEnabled(true);
    }
    catch ( RtMidiError &error ) {
        QMessageBox::warning(this, "Error", "A problem has occurred attempting to disconnect to the MIDI port.");
        error.printMessage();
    }
}

void MotasUtility::sendMIDISysExCommand(QString name, char msg1, char msg2)
{
     Q_UNUSED(name);
    if (this->midiOut->isPortOpen())
    {
        QVector<quint8> message;
        message.append(MIDI_stuff::BYTE_SYSEX_START);
        message.append(MIDI_stuff::BYTE_SYSEX_ID1);
        message.append(MIDI_stuff::BYTE_SYSEX_ID2);

        message.append(MIDI_stuff::BYTE_SYSEX_ID3);
        message.append( ui->spinBoxMotasModel->value());
        message.append(ui->spinBoxChannel->value() - 1);

        message.append( 0);
        message.append(msg1);
        message.append(msg2);

        message.append(MIDI_stuff::BYTE_SYSEX_END);

        std::vector<unsigned char> m = message.toStdVector();
        countDownTimerMIDIOut = 6;
        this->midiOut->sendMessage( &m );

        qDebug() << "MidiSettingsDialog:  sending SYSEX command: " << name;

    } else
    {
        QMessageBox::warning(this, "Error", "No MIDI out port!.");
        ui->checkBoxSendBitmap->setChecked(false);
        ui->checkBoxRequestOscilloscopeData->setChecked(false);
    }
}


void MotasUtility::on_checkBoxSendBitmap_clicked()
{
    if (this->ui->checkBoxSendBitmap->isChecked())
    {
        ui->checkBoxRequestOscilloscopeData->setChecked(false);
     //   sendMIDISysExCommand("bitmap cmd", MIDI_SYSEX_REQUEST_BITMAP, 0);
        this->screenCapture->show();
        //this->screenCapture->makeActive();
    }
    /*else
    {
        this->screenCapture->resetWindow();
    }
*/
}


void MotasUtility::on_pushButtonFindDevices_clicked()
{
    numBytesReceived = 0;
    ui->MIDIopen_pushButton->setText("Connect");
     ui->pushButtonAbort->setEnabled(!true);
    ui->comboBoxOutMIDI->setEnabled(!false);
    ui->MIDIin_comboBox->setEnabled(!false);
    updateMIDI(true);
    //ui->checkBoxMidiConnected->setChecked(!true);
    isConnected = false;
    ui->MIDIopen_pushButton->setEnabled(true);
    reportValues = TRUE;
}

void MotasUtility::on_pushButtonAbort_clicked()
{
    sendMIDISysExCommand("abort cmd", MIDI_SYSEX_ABORT_TRANSFER, 0);
}

void MotasUtility::on_comboBoxOutMIDI_currentTextChanged(const QString &arg1)
{
    /*
    if (arg1.contains("motas", Qt::CaseInsensitive))
    {
        ui->spinBoxMIDIdelay->setValue(1200);
    } else
    {
        ui->spinBoxMIDIdelay->setValue(1200);
    }*/
}





void MotasUtility::on_checkBoxRequestOscilloscopeData_clicked()
{
    if (this->ui->checkBoxRequestOscilloscopeData->isChecked())
    {
        ui->checkBoxSendBitmap->setChecked(false);
       // sendMIDISysExCommand("oscilloscope cmd", MIDI_SYSEX_REQUEST_OSCILLOSCOPE_TRACE, 0);
          this->oscilloscopeCapture->show();
    }
}

void MotasUtility::on_pushButton_clicked()
{
    sendMIDISysExCommand("bulk FLASH data cmd", MIDI_SYSEX_REQUEST_BULK_FLASH_DATA_SEND, 0);
}

void MotasUtility::on_pushButtonSendBulk_clicked()
{

    QString fileName;

    fileName =  QFileDialog::getOpenFileName(this,"load bulk data",  fileNameLocationDefault + "*.dat");

    if (fileName.isEmpty())
        return;


    QFileInfo fileInfo(fileName);
    fileNameLocationDefault = fileInfo.absolutePath() ;


    QFile f(fileName);
    qDebug() << fileName << "file size: " << f.size();


    quint32 fileSize = MASS_FLASH_TRANSFER_LENGTH_BYTES;


    if ((f.size()) != fileSize)
    {
        qDebug() <<  "file size: " << f.size() << " != " << fileSize;
        QMessageBox::warning(this, "Error", "ERROR IN FILE SIZE");
        f.close();
    } else
    {

        quint32 bytesToSend = 0;
        quint8 bulkDataBuffer[fileSize];
        f.open(QIODevice::ReadOnly);

        f.read((char*) bulkDataBuffer, fileSize);
        f.close();

        quint8* outBuffer = new quint8[4096*96 * 2];


        for (int i = 0; i < 8; i++)
        {

            qint16 offset = i * 96;
            bytesToSend  = motasFirmware->prepareSysExPacketsFromUnpacked(outBuffer, &bulkDataBuffer[4096*offset],
                    4096*96,  MIDI_SYSEX_RECEIVE_BULK_FLASH_DATA, offset / 128, offset % 128, 96,
                    ui->spinBoxMotasModel->value(), ui->spinBoxChannel->value() -1);
            if (bytesToSend == 0)
                break;
            qDebug() <<  "bytes to send: " << bytesToSend;


            if (this->sendDataOverMIDIDirect(outBuffer, bytesToSend) == 0)
                break;
            QThread* th = new QThread;
            th->usleep(500000);
            delete th;
        }

        delete outBuffer;
        outBuffer = 0;

    }
}




void MotasUtility::resendMIDIcommands()
{
    if (ui->checkBoxRequestOscilloscopeData->isChecked() && displayedOscilloscopeTrace == TRUE)
    {
        sendMIDISysExCommand("oscilloscope cmd", MIDI_SYSEX_REQUEST_OSCILLOSCOPE_TRACE, 0);
    }
    else if (ui->checkBoxSendBitmap->isChecked()) // ask for another bitmap
    {
        sendMIDISysExCommand("bitmap cmd", MIDI_SYSEX_REQUEST_BITMAP, 0);
    }
}


void MotasUtility::on_checkBoxSmoothImage_clicked()
{
    if (ui->checkBoxSmoothImage->isChecked())
    {
        screenCapture->setSmoothImage(true);
    } else
    {
        screenCapture->setSmoothImage(!true);
    }
}

void MotasUtility::on_pushButtonRequestSetup_clicked()
{
     sendMIDISysExCommand("global setup data cmd", MIDI_SYSEX_REQUEST_GLOBALS, 0);
}





void MotasUtility::on_pushButtonSendSetup_clicked()
{
    QString fileName;

    fileName =  QFileDialog::getOpenFileName(this,"load setup data",  fileNameLocationDefault + "*.setup");

    if (fileName.isEmpty())
        return;


    QFileInfo fileInfo(fileName);
    fileNameLocationDefault = fileInfo.absolutePath() ;


    QFile f(fileName);
    qDebug() << fileName << "file size: " << f.size();


    quint32 fileSize = SETUP_DATA_LENGTH_BYTES;


    if ((f.size()) != fileSize)
    {
        qDebug() <<  "file size: " << f.size() << " != " << fileSize;
        QMessageBox::warning(this, "Error", "ERROR IN FILE SIZE");
        f.close();
    } else
    {

        quint32 bytesToSend = 0;
        quint8 bulkDataBuffer[fileSize];
        f.open(QIODevice::ReadOnly);

        f.read((char*) bulkDataBuffer, fileSize);
        f.close();

        quint8* outBuffer = new quint8[SETUP_DATA_LENGTH_BYTES * 2];



        bytesToSend  = motasFirmware->prepareSysExPacketsFromUnpacked(outBuffer, &bulkDataBuffer[0],
                SETUP_DATA_LENGTH_BYTES,  MIDI_SYSEX_GLOBALS_TRANSFER, 0, 0, 0,
                ui->spinBoxMotasModel->value(), ui->spinBoxChannel->value() - 1);

        qDebug() <<  "bytes to send: " << bytesToSend;


        this->sendDataOverMIDIDirect(outBuffer, bytesToSend);



        delete outBuffer;
        outBuffer = 0;

    }
}

void MotasUtility::on_TabWidget_currentChanged(int index)
{
    Q_UNUSED(index);
    updateCurrentDisplayData();
}

void MotasUtility::on_tabWidgetSingleOrBank_currentChanged(int index)
{
    Q_UNUSED(index);
    updateCurrentDisplayData();
}


