/*

    Copyright (c) 2017 Motas Electronics Limited (J M HAYES)
    www.motas-synth.uk


    Permission is hereby granted, free of charge, to any person
    obtaining a copy of this software and any associated documentation files
    (the "Software"), to deal in the Software without restriction,
    including without limitation the rights to use, copy, modify, merge,
    publish, distribute, sublicense, and/or sell copies of the Software,
    and to permit persons to whom the Software is furnished to do so,
    subject to the following conditions:

    The above copyright notice and this permission notice shall be
    included in all copies or substantial portions of the Software.

    Any person wishing to distribute modifications to the Software is
    asked to send the modifications to the original developer so that
    they can be incorporated into the canonical version.  This is,
    however, not a binding provision of this license.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
    ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
    CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
    WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


 *
 * */

#ifndef UTILITIES_H
#define UTILITIES_H
#include "QString"
#include "RtMidi/RtMidi.h"

class Utilities
{
public:
    Utilities();
    void generateSVGIcons(QString fileNameFontDefault);
    void convertBitmap(QString fileNameBitmapDefault);
    static qint16 convertMIDIbytes(quint8 currentByte, quint32 i);
    static quint32 lowLevelCreateMIDIStream(quint8* buf, quint32 size,  quint8 type, quint8 startStopContinue,
            quint8 msg0, quint8 msg1, quint8 msg2, quint8* outBuf, quint8 model, quint8 channel);

    static quint32 processBufferForMIDISend(char* outBuffer, char* buffer, quint32 originalDataLength);

    static void write32(quint32 val, int offset, quint8* buffer);

    static quint32 read32(int offset, quint8* buffer);

    static QVector<quint8> prepareSysExPacket(int j, char* buffer, quint32 packetSize, quint32 fileLength, qint32 lastPacket, int type, quint8 msg0, quint8 msg1, quint8 msg2, quint8 modelNum, quint8 channel);

    static void memcpyReverse32(char* out, char* in);


    static void memcpyForward32(char* out, char* in);


    void setMotasModel(int modelNum);


};


namespace MIDI_stuff
{
    const quint8 BYTE_SYSEX_START   = 0xF0;
    const quint8 BYTE_SYSEX_END     = 0xF7;


    const quint8 BYTE_SYSEX_ID1     = 0x00; // Motas Electronics Limited assigned MIDI SYSEX IDs
    const quint8 BYTE_SYSEX_ID2     = 0x21; // Motas Electronics Limited assigned MIDI SYSEX IDs
    const quint8 BYTE_SYSEX_ID3     = 0x2F; // Motas Electronics Limited assigned MIDI SYSEX IDs

    const quint8 BYTE_SYSEX_PRODUCT_ID_1  = 0x06; // product identifier (motas 5, 6,...)


    const quint8 BYTE_CONTROL_CHANGE = 0xB0;

    // NRPN method
    const quint8 BYTE_CONTROL_SET_PAGE = 0x63; // undefined controller
    const quint8 BYTE_CONTROL_SET_PARAMETER = 0x62; // undefined controller
    const quint8 BYTE_CONTROL_SET_VALUE_MSB = 0x06; // data
    const quint8 BYTE_CONTROL_SET_VALUE_LSB = BYTE_CONTROL_SET_VALUE_MSB + 32;

}



#endif // UTILITIES_H
